;==========================================================================
;==								         ==
;==			        STUFIT				         ==
;==========================================================================

;Utility to move designated files to the inner tracks of a disk.
;
;Disassembled and heavily hacked, with most of the significant changes
;annotated with the "v1.x" comments.
;
;Incorporating DOS v3.x/4.x patches from:
;	Hylton Boothroyd, University of Warwick, 890710
;See his notes at the end.
;
; - No need to reset DTA for each file.
;   Just doing it once now (at Find_First)
;
; - Confirmed via tests, no need to "force DOS next free space"
;   at every run (as in Hylton's original patch).
;   Just doing it once now (at Check_DOS_Version)
;
;I've noticed the program halts when it hits a file too big to relocate
;(e.g., free space is less than target file size).
;This is as it should be .. but halting right there is not so very smart:
;we should be smart enough to go on beyond that too-big file in case
;there are smaller ones that WILL fit.
;
;In the meantime, this utility will work best if you use a directory sorting
;utility (like Norton's Utilities) to sort in order of file size
;(smallest files first).
;
;This enables the "Find First" and "Find Next" functions to pick up
;files in an order most likely to maximize the number of files that can
;be moved to the innermost tracks.  (Assuming you have a free space
;limitation that prohibits moving ALL files.)
;
;
;We're also sticking with the sheer brute-force method of forcing
;our "next free space" to disk end (via creating a temporary free-space
;holding file that gobbles ALL the free space on the disk .. except for
;the space required for the target file copy).
;
;There oughtta be a way to fool DOS into finding free space working
;backwards from the inside tracks, or from the last new file we created
;during relocation.  Unfortunately, I can't think of any way!
;
;
;David Kirschbaum
;Toad Hall
;kirsch@braggvax.ARPA

CR	equ	0DH
LF	equ	0AH

CSEG	SEGMENT PUBLIC PARA 'CODE'
	ASSUME	CS:CSEG, DS:CSEG, ES:CSEG

		org	0
cseg_org	label	byte			;needed to help MASM
						;figure buffer size	v1.1

		org	80H
cmdline		label	byte			;v1.1

		org	100h

StufIt		proc	near

		jmp	Start

;v1.2	Some data previously scattered around the program
;	consolidated here.

ourDTA	label	byte					;working dta
	db	'dta2dta2dta2dta2dta2dta2dta2dt'
;DTA+30							;file name psn
							;within DTA
	db	'a2'
	db	'dta2dta2dta2dta2dta2dta2dta2dta2'
	db	'dta2dta2dta2dta2dta2dta2dta2dta2'
	db	'dta2dta2dta2dta2dta2dta2dta2dta2'

targetname	label	byte				;quite a buffer!
	db	'filenamefilenamefilenamefilename'
	db	'filenamefilenamefilenamefilename'
	db	'filenamefilenamefilenamefilename'
	db	'filenamefilenamefilenamefilename'

;for current filename display
namelen	dw	0					;length
nameptr	dw	targetname				;ptr to filename


StufIt		endp

;==========================================================================
;			       SUBROUTINE
;==========================================================================


err2	db	'-File not found-',CR,LF,'$'
err3	db	'-Path not found-',CR,LF,'$'
err4	db	'-Too many open files (no handles left)-',CR,LF,'$'
err5	db	'-Access denied-',CR,LF,'$'
err6	db	'-Invalid handle-',CR,LF,'$'
err0C	db	'-Invalid access code-',CR,LF,'$'
err11	db	'-Not same device-',CR,LF,'$'
err12	db	'-No more files-',CR,LF,'$'
err1C	db	'-Write error-',CR,LF,'$'		;v1.1
errunk	db	'-Unrecognized error code-'
crlf$	db	CR,LF,'$'			;share this	v1.1

;Vastly tightened
;Enter with:
;  AX = file error
;  DX = first error message

Rpt_Error	proc	near			;v1.1

	mov	bx,ax			;save error value		v1.1
	mov	si,dx			;and msg ptr			v1.1
	mov	dx,offset crlf$		;always a new line		v1.1
	mov	ah,9			;display msg			v1.1
	int	21H			;				v1.1
	mov	dx,si			;first msg ptr			v1.1
	mov	ah,9			;display msg			v1.1
	int	21H			;				v1.1
	mov	ax,bx			;restore error			v1.1

	mov	dx,offset err2		;'file not found'
	cmp	al,2
	jz	Pr_ErrMsg

	mov	dx,offset err3		;'Path not found'
	cmp	al,3
	jz	Pr_ErrMsg

	mov	dx,offset err4		;'Too many open files'
	cmp	al,4
	jz	Pr_ErrMsg

	mov	dx,offset err5		;'Access denied'
	cmp	al,5
	jz	Pr_ErrMsg

	mov	dx,offset err6		;'Invalid handle'
	cmp	al,6
	jz	Pr_ErrMsg

	mov	dx,offset err0C		;'Invalid access code'
	cmp	al,0CH
	jz	Pr_ErrMsg

	mov	dx,offset err11		;'Not same device'
	cmp	al,11H
	jz	Pr_ErrMsg

	mov	dx,offset err12		;'No more files'
	cmp	al,12H
	jz	Pr_ErrMsg

	mov	dx,offset err1C		;'Write error'			v1.1
	cmp	al,1CH			;				v1.1
	jz	Pr_ErrMsg		;				v1.1

	mov	dx,offset errunk	;'Unknown error'

Pr_ErrMsg:
	mov	bx,ax			;save error value		v1.1
	mov	ah,9			;display msg
	int	21H
	mov	ax,bx			;restore error value		v1.1
	stc				;always return CF set		v1.1
	ret

Rpt_Error	endp

;==========================================================================
;			       SUBROUTINE
;==========================================================================

;Wonder why he bothered with these default values?

bytesPerSector		dw	0		;was 200h		v1.2
sectorsPerCluster	dw	0		;was 8			v1.2
bytesPerCluster		dw	0		;was 1000h		v1.2

nrClusters		dw	0		;was 0A1AH		v1.2
avail$			db	'avail: $'	;			v1.1
availClusters		dw	0


Get_FreeSpace	proc	near

	mov	ah,36h			;get free space
	xor	dx,dx			;current drive			v1.1
	int	21h
	jnc	Loc_12			;got it ok

	mov	dx,offset avail$	;'avail: '
	jmp	Rpt_Error		;display error msg,		v1.1
					;return from there CF set

Loc_12:
	mov	availClusters,bx	;available cluster count
	mov	nrClusters,dx		;total clusters
	mov	bytesPerSector,cx	;bytes per sector
	mov	sectorsPerCluster,ax	;sectors per cluster
	mul	cx
	mov	bytesPerCluster,ax	;bytes per cluster
	clc				;return CF clear
	ret

Get_FreeSpace		endp


;==========================================================================
;			       SUBROUTINE
;==========================================================================

getfirst$	db	'getfirst: $'		;v1.1
getnext$	db	'getnext: $'		;v1.1

;Code for Find First and Find Next

Find_File	proc	near

Find_Next:

;v1.2	I wonder why we keep resetting the DTA?
;	We did it once at Find_First .. why again and again?
;	Trying this without the DTA reset

Comment	~
	mov	dx,offset ourDTA	;working DTA
	mov	ah,1AH			;set DTA to DS:DX
	int	21h
	jc	FindNext_Fail		;failed
Comment	ends	~

	mov	ah,4FH			;find next filename
	int	21h

;v1.1	jc	FindNext_Fail		;failed
;Wrongo!  Find Next does NOT use the CF to indicate success/failure.
;Instead, AX returns a value (0 if found).

	or	ax,ax			;found it?
	jnz	FindNext_Fail		;nope, die

	 mov	si,nameptr		;get ptr to filename
	 jmp	short Find_Common	; and continue


FindNext_Fail:
	mov	dx,offset getnext$	;'getnext: '
	jmp	Rpt_Error		;display file error		v1.1
					;return from there CF set

;Enter here for the Find First.

Find_First:
	mov	dx,offset ourDTA	;working DTA
	mov	ah,1AH			;set DTA to DS:DX
	int	21h
	jc	FindFirst_Fail		;failed				v1.1

	mov	dx,offset targetname	;filename buffer
	xor	cx,cx			;no special attribs
	mov	ah,4Eh			;find first
	int	21h

;v1.1	jnc	Loc_19			;found it
;Wrongo!  Find First doesn't use the carry flag to indicate
;success or failure!  Results are in AX!

	or	ax,ax			;0 means success		v1.1
	jnz	FindFirst_Fail		;didn't find it			v1.1
	
	mov	si,offset targetname-1	;filename buffer-1
	mov	cx,namelen		;name length			v1.1
	add	si,cx			;bump filename ptr also		v1.1
	std				;move backwards

LocLoop_20:
	lodsb				;snarf target filename char
	cmp	al,'\'			;part of a path?
	je	NameStart_21		;yep
	cmp	al,'/'			;could be using this as path
	je	NameStart_21		;yep
	cmp	al,':'			;how about a drive separator?
	je	NameStart_21		;yep
	loop	LocLoop_20		;just a name char

	dec	si			;adjust from last lodsb

NameStart_21:
	mov	nameptr,si		;save ptr to filename start

;Common code for Find_First and Find_Next

Find_Common:
	mov	di,si			;target filename start
	mov	si,offset ourDTA+30	;30 bytes into working DTA
	add	di,2			;bump name ptr
	cld				;insure fwd
	mov	cx,0Dh			;move 13-byte name
					;from DTA+30 to DI^

LocLoop_23:
	lodsb
	stosb
	or	al,al			;terminating AsciiZ?		v1.1
	loopnz	LocLoop_23		;nope

	sub	di,offset targetname+1 ;last char - buffer start
	mov	namelen,di		;save name length

	mov	dx,offset targetname	;display filename...
	mov	cx,di			;bytes to write			v1.1
	mov	bx,1			;file handle (STDOUT)
	mov	ah,40H			;write to file/device
	int	21H

	mov	dx,offset crlf$		;CR/LF
	mov	ah,9			;display msg
	int	21H
	clc				;return CF clear
	ret

FindFirst_Fail:
	mov	dx,offset getfirst$	;'getfirst'
	jmp	Rpt_Error		;report file error,		v1.1
					;return from there CF set

Find_File	endp


;==========================================================================
;			       SUBROUTINE
;==========================================================================

filesizeLo	dw	0
filesizeHi	dw	0
flen$		db	'getflen: $'		;error msg		v1.1

Get_TgtFile_Size	proc	near

	mov	dx,offset targetname	;filename			v1.1
	mov	ax,3D00H		;open file, read only
	int	21h
	jc	GetSize_Failed		;open failed

	mov	bx,ax			;handle into BX
	xor	cx,cx			;0 offset
	xor	dx,dx			;ditto
	mov	ax,4202H		;move file ptr from end		v1.1
	int	21H			; (e.g., to end, produces size)
	jc	GetSize_Failed		;move ptr failed

	mov	filesizeHi,dx		;target filesize.hi
	mov	filesizeLo,ax		;target filesize.lo
	mov	ah,3EH			;close file
	int	21h			;(BX still handle)
	jc	GetSize_Failed		;failed
	 ret				;return CF clear		v1.1

GetSize_Failed:
	mov	dx,offset flen$		;'flen: '
	jmp	Rpt_Error		;display file error,		v1.1
					;return from there CF set
Get_TgtFile_Size	endp


;==========================================================================
;			       SUBROUTINE
;==========================================================================

;fileclusters	dw	0	v1.2 never used
clustercnt	dw	0

needlen$	db	'needlen: $'			;error msgs	v1.1
zeroDiv$	db	'zero divide error',CR,LF,'$'
insufSpc$	db	'not enough disk space',CR,LF,'$'


Check_Space_Remaining	proc	near

	mov	bx,offset zeroDiv$	;'Zero divide error'		v1.1
;v1.2	cmp	bytesPerCluster,0	;0 bytes per cluster?
;v1.2	jz	CN_CommonErr		;yep, display msg, die		v1.1
	mov	cx,bytesPerCluster	;snarf bytes per cluster	v1.2
	jcxz	CN_CommonErr		;0 .. zero divide error		v1.2

	mov	dx,filesizeHi		;target filesize.hi
	mov	ax,filesizeLo		;target filesize.lo
;v1.2	div	bytesPerCluster		;produces AX=nr clusters
	div	cx			;/ bytes per cluster		v1.2
					; = nr clusters (in AX)
	or	dx,dx			;any remainder?			v1.1
	je	Loc_27			;nope
	 inc	ax			;yep, round quotient up		v1.1
Loc_27:

;v1.2	We never use this variable again .. why keep it?
;v1.2	mov	fileclusters,ax		;save result (file size
					;in clusters)
	cmp	ax,availClusters	;do we have that many avail?
	jbe	Got_Suff_Space		;yep, ok

	mov	bx,offset insufSpc$	;'Not enough disk space'	v1.1

CN_CommonErr:				;common err routine		v1.1
	mov	dx,offset needlen$	;'needlen: ' error msg
	mov	ah,9			;display msg
	int	21h

	mov	dx,bx			;BX=second err msg^		v1.1
	mov	ah,9			;display msg
	int	21h
	mov	al,63h			;terminal error			v1.1
	stc				;return CF set			v1.1
	ret				;				v1.1

Got_Suff_Space:
	neg	ax			;flip bits
	add	ax,availClusters	;free clusters remaining
	mov	clustercnt,ax		;save as free-space holding
					; file size (free space - target
					; file size)
	clc				;return CF clear
	ret

Check_Space_Remaining		endp

;==========================================================================
;			       SUBROUTINE
;==========================================================================

;Text to stuff in free-space holding file:

data_638	db	'Ta-da.'
data_63E	db	'Finis.'

fileptrLo	dw	0		;file ptr.lo			v1.1
fileptrHi	dw	0		;file ptr.hi			v1.1

dumfile$	db	'dumfile: $'	;error msg			v1.1

;Create a temporary file that takes all free space
;except for that space needed for the target file copy.

Create_TempFile		proc	near

	mov	ax,clustercnt		;nr of clusters avail on disk
					;(minus our target file size)
	or	ax,ax			;if it's zero ...		v1.1
	jz	Loc_ret_33		; just return			v1.1

	mul	bytesPerCluster		; * bytes per cluster
	mov	fileptrHi,dx		;save as file ptr.hi and
	mov	fileptrLo,ax		; ptr.lo for our temp file

	mov	dx,offset gobbige	;'gobbige.zzz'			v1.1
	mov	cx,20h			;attribute
	mov	ah,3Ch			;create file
	int	21h

	mov	bx,ax			;save gobbige handle
	jc	FileError_35		;failed, die

	mov	dx,offset data_638	;'Ta-da.'
	mov	cx,6			;write 6 bytes
	mov	ah,40h			;write to file/device
	int	21h			;(gobbige.zzz)
	jc	FileError_34		;write failed

	mov	cx,fileptrHi		;get file pointer.hi
	mov	dx,fileptrLo		; .. and .lo
	mov	ax,4200H		;move gobbige file ptr from file start
	int	21h			; to force file size to fill all
					; free space (minus target file size)
	jc	FileError_34		;failed, die

;v1.1	mov	ax,6
;v1.1	neg	ax
	mov	ax,0FFF9H		;same thing as neg 6		v1.1
	cwd				;doubleword it
	mov	cx,dx			;file offset.hi
	mov	dx,ax			;file offset.lo

	mov	ax,4201H		;move file ptr from current psn
	int	21h			;(e.g., 6 bytes back)
	jc	FileError_34		;failed

	mov	dx,offset data_63E	;'Finis.'
	mov	cx,6			;write 6 bytes
	mov	ah,40h			;write to file/device
	int	21h			;(gobbige.zzz's very end)
	jc	FileError_34		;write failed

	mov	ah,3Eh			;close temp file handle
	int	21h			;(gobbige.zzz)
	jc	FileError_35		;failed				v1.1

Loc_ret_33:				;				v1.1
	ret				;closed ok, CF clear		v1.1


FileError_34:
	mov	dx,ax			;save file error		v1.2
	mov	ah,3EH			;close file
	int	21h			;(handle still in BX)
	mov	ax,dx			;restore file error		v1.2

FileError_35:
	mov	dx,offset dumfile$	;'dumfile: '
	jmp	Rpt_Error		;display file error,		v1.1
					;return from there, CF set

Create_TempFile		endp


;==========================================================================
;			       SUBROUTINE
;==========================================================================

tgthandle	dw	0
temphandle	dw	0

openErr$	db	'open: $'		;error msgs		v1.1
createErr$	db	'create: $'		;			v1.1
dateErr$	db	'redate: $'		;			v1.1
writeErr$	db	'CPY: Destination write error',CR,LF,'$'	;v1.1

;Enters with SI -> AsciiZ filename
;  si -> targetname		file name buffer
;  di -> offset zztop		'zz.top'

Copy_FileData	proc	near

	mov	dx,offset targetname	;filename ptr			v1.1
	mov	ax,3D00H		;open file, AL=mode		v1.1
	int	21h
	mov	dx,offset openerr$	;'open:'			v1.1
	jb	FileError_37A		;open failed, die		v1.1

	mov	tgthandle,ax		;save file handle

	mov	dx,offset zztop		;'zz.top'			v1.1
	xor	cx,cx			;normal attribs			v1.1
	mov	ah,3Ch			;create file
	int	21h
	jnc	Loc_38			;created ok

	mov	dx,ax			;save error
	mov	bx,tgthandle		;get opened file handle
	mov	ah,3EH			;close file
	int	21h			;(ignore errors)
	mov	ax,dx			;restore create error
	mov	dx,offset createErr$	;'create: '			v1.1
FileError_37A:				;common code			v1.1
	jmp	Rpt_Error		;display file error,		v1.1
					;return from there CF set


Loc_38:	mov	temphandle,ax		;save created file's handle

Lup_0733:
	mov	bx,tgthandle		;get opened file handle
	mov	cx,BUFFSIZE		;bytes to read			v1.1
	mov	dx,offset filebuff	;read buffer start		v1.1
	mov	ah,3Fh			;read file/device
	int	21h
	jb	CloseUp_45A		;read failed, close up		v1.1

	or	ax,ax			;read any bytes?		v1.1
	jz	Eof_44			;nope, EOF			v1.1

	mov	bx,temphandle		;get created file handle
	mov	cx,ax			;bytes to write
	mov	ah,40h			;write to file/device
	int	21h
	jc	CF_Write_Error		;failed				v1.1

	cmp	ax,cx			;wrote all we asked?
	je	Lup_0733		;yep, keep going		v1.1

CF_Write_Error:
	call	Close_BothFiles		;close both files
	mov	dx,offset writeErr$	;'Write error '
	mov	al,1CH			;fake write error		v1.1
	jmp	Rpt_Error		;error msgs,			v1.1
					;return CF set

;EOF, file copied.  Change new file date/time to old file date/time.

Eof_44:	mov	bx,tgthandle		;get opened file handle
	mov	ax,5700H		;get file date/time		v1.1
	int	21h
	jc	CloseUp_45A		;failed, close up		v1.1

	mov	bx,temphandle		;get created file's handle
	mov	ax,5701H		;set file date/time		v1.1
	int	21h			;(same as orig file)
	jc	CloseUp_45A		;failed				v1.1
	 call	Close_BothFiles		;close up			v1.1
	 ret				;return, CF per close		v1.1


CloseUp_45A:				;common code			v1.1
	mov	dx,ax			;save error code		v1.2
	call	Close_BothFiles		;close both files
	mov	ax,dx			;restore error			v1.2
	mov	dx,offset dateErr$	;'redate: '			v1.1
	jmp	Rpt_Error		;display file error,		v1.1
					;return from there CF set

Copy_FileData		endp


;==========================================================================
;			       SUBROUTINE
;==========================================================================

Close_BothFiles		proc	near

	mov	bx,tgthandle		;file handle
	mov	ah,3EH			;close file
	int	21h
	mov	bx,temphandle		;temp file handle
	mov	ah,3EH			;close file
	int	21h
	ret

Close_BothFiles		endp


;==========================================================================
;			       SUBROUTINE
;==========================================================================

delfile$	db	'delfile: $'		;error msg		v1.1

Delete_File	proc	near

	mov	ah,41h			;delete file
	int	21h			;(DS:DX -> name)
	jc	Delete_Failed		;failed				v1.1
	ret

Delete_Failed:
	mov	dx,offset delfile$	;'delfile: '
	jmp	Rpt_Error		;report file error,		v1.1
					;return from there CF set

Delete_File		endp


;==========================================================================
;			       SUBROUTINE
;==========================================================================

renfile$	db	'renfile: $'		;error msg		v1.1

;Rename our temporary file ('zz.top') to the old filename.

Rename_File	proc	near

	mov	dx,offset zztop		;'zz.top'			v1.1
	mov	di,offset targetname	;old filename			v1.1
	mov	ah,56H			;rename file
	int	21h			;@DS:dx to @ES:di
	jb	Rename_Failed		;failed				v1.1
	 ret				;ok, CF clear

Rename_Failed:
	mov	dx,offset renfile$	;'renfile'
	jmp	Rpt_Error		;display file error,		v1.1
					;return from there CF set

Rename_File		endp


;==========================================================================
;			       SUBROUTINE
;==========================================================================


zztop	db	'zz.top',0		;temp data file name
gobbige	db	'gobbige.zzz',0		;free-space holding file name


Start	proc	near
	call	Check_DOS_Version	;get DOS version
	jc	Stop_53			;too early, die

	call	Get_FreeSpace		;check disk free space
	jc	Stop_53			;insufficient, die

	call	Process_CmdLine		;check PSP cmdline
	jc	Stop_53			;wrong, die

	call	Find_First		;set DTA, try to find it
	jc	Stop_53			;failed, die

;Loop over all files here

Lup_0875:
	call	Get_TgtFile_Size	;get target file size
	jc	Stop_53			;failed

	call	Check_Space_Remaining	;check target file size against
					; avail free space
	jc	Stop_53			;failed, die

	call	Create_TempFile		;create a file to fill all avail
					; free space (minus target file size)
	jc	Stop_53			;failed, die

;v1.1	cli				;hmmmm .. why this?

	call	Copy_FileData		;copy target file data		v1.1
					; to temporary data file
	jc	Stop_53			;failed, die

;copy went ok, delete old version

	mov	dx,offset targetname	;target file name buffer
	call	Delete_File		;delete it
	jc	Stop_53			;failed, die

;delete the bogus freespace holding file

	mov	dx,offset gobbige	;'gobbige.zzz'
	call	Delete_File		;delete freespace file
	jc	Stop_53			;failed, die

;rename new file to original name, change date.

	call	Rename_File		;rename zz.top to target filename
	jc	Stop_53			;failed, die

;v1.1	sti				;hmmmm???

	call	Find_Next		;find next target file
	jnc	Lup_0875		;went ok, keep going		v1.1

;All files done

	xor	al,al			;ERRORLEVEL 0			v1.1

Stop_53:				;				v1.1
;v1.1	sti				;ints back on (????)
	mov	ah,4CH			;terminate			v1.1
	int	21H			;				v1.1

Start	endp


	even				;v1.1

filebuff label	byte

BUFFSIZE	=	(offset cseg_org + 0F000H) - offset filebuff	;v1.2

;v1.1	Moved logo and usage down to buffer space

	db	'V1.2TH, 6.13.85 Zider Brothers, San Francisco',0
	db	'Toad Hall / Boothroyd Update 26 Jul 89',0

usage$	db	'Syntax:  STUFIT filename.ext ',CR,LF,LF
	db	'         [\][path\]*.*',CR,LF,LF
;v1.2	db	'Path, but no drive: files must be'
;v1.2	db	' on default drive. ',CR,LF,'$'
	db	'You MUST specify a filename ("*.*" wildcard is OK).',CR,LF
	db	'You may include paths, but NO DRIVE!',CR,LF
	db	'Files must be on default drive.',CR,LF,'$'

;==========================================================================
;			       SUBROUTINE
;==========================================================================

;v1.2	Moved this down to buffer space
;	since it's only used at startup
;	(prior to any file reads)

dosver$	db	'Incorrect DOS version: need 2.0 '
	db	'or higher',CR,LF,'$'

Check_DOS_Version	proc	near

	mov	ah,30H			;get DOS version
	int	21h
	cmp	al,2			;2.0 or above?
	jae	Dos_Ok			;yep, fine

	mov	dx,offset dosver$	;'Incorrect DOS version...'
	mov	ah,9			;display msg
	int	21h
	mov	al,63h			;dummy error msg		v1.1
	stc
	ret

Dos_Ok:

;v1.1	Patches for DOS 3.x/4.x per HB (but tweaked)
;	See comments at end
;	Decided to do this just once (while we know what
;	DOS version we have, rather than at every file
;	(like HB patched it to do).

	je	HB_Done			;v2.x, no further action	v1.1

	push	DS			;save some regs			v1.1

	xor	ah,ah			;clear msb			v1.1
	add	al,19H			;add in magic offset now	v1.2
	mov	di,ax			;DI -> next_free word		v1.1
					; (cluster at which to
					; start next free space search)

;v1.1	We use the DOS version now in DI) as an offset
;into the drive parameter block to force the "next free block"
;to the very end of the drive (e.g., innermost block).

	mov	ah,32H			;get drive parm block
					;(undocumented DOS function)
	xor	dl,dl			;current drive
	int	21H

;DS:BX now points to the drive parameter block.
;DI holds our DOS version (high word, like 3 or 4) + the table offset
; so it's pointing to 1CH or 1DH)
;
;Here's what the last part of the drive parm table looks like.
;
;  First, DOS 3.x:
;
; 17h	BYTE	FFh = block must be rebuilt, 00h indicates block accessed
; 18h	DWORD	address of next device block, offset = FFFFh indicates last
; 1Ch	WORD	cluster at which to start search for free space when writing
; 1Eh	WORD	number of free clusters on drive, FFFFh = unknown???
;
;  Now, DOS 4.0 (slightly different):
;
; 18h	BYTE	FFh = block must be rebuilt, 00h indicates block accessed
; 19h	DWORD	address of next device block, offset = FFFFh indicates last
; 1Dh	WORD	cluster at which to start search for free space when writing
; 1Fh	WORD	number of free clusters on drive, FFFFh = unknown???
;
;
;We poke a new "cluster at which to start search for free space"
;into the appropriate table position (either offset 1CH (v3.x)
;or 1DH (v4.0)

	mov	word ptr [BX+DI],2		;force DOS to start search
						;here!

	pop	DS				;restore

HB_Done:
	clc
	ret

Check_DOS_Version		endp

;==========================================================================
;			       SUBROUTINE
;==========================================================================
;v1.2	Moved down to buffer space
;	(Only used at startup)

Process_CmdLine		proc	near

	xor	ax,ax			;clear msb
	mov	si,offset cmdline	;PSP cmdline length byte
	lodsb				;snarf length byte		v1.1
	mov	cx,ax			;move into CX			v1.1
	jcxz	Loc_16			;usage, halt			v1.1

	mov	ax,offset ourDTA	;keep this handy
					;move filename from DS:81H
					;(PSP cmdline)
	mov	di,ax			;to our DTA buffer
	mov	bx,cx			;save the length		v1.1
	rep	movsb
	mov	si,ax			;SI -> ourDTA			v1.1
	mov	di,offset targetname	;move name into here		v1.1
	mov	cx,bx			;nr chars in cmdline		v1.1

	mov	ah,20H			;handy space constant		v1.1
LocLoop_13:
	lodsb
	cmp	al,ah			;gobble leading spaces		v1.1
	je	Loc_14

	cmp	al,CR			;CR terminates
	je	Loc_15
	 stosb
Loc_14:
	 loop	LocLoop_13

Loc_15:
	mov	byte ptr [di],0		;AsciiZe the filename

	mov	ax,di			;last byte +1
	sub	ax,offset targetname	;- start
	mov	namelen,ax		;= name length
	or	ax,ax			;any length?			v1.1
	je	Loc_16			;nope, error
	 clc				;return CF clear
	 ret

Loc_16:
	mov	dx,offset usage$	;'Syntax: ...'
	mov	ah,9			;display msg
	int	21h
	xor	al,al			;errorlevel 0			v1.1
	stc				;return CF set
	ret

Process_CmdLine		endp



CSEG	ends
	end	StufIt

From: Hylton Boothroyd <bsrdp%cu.warwick.ac.uk@NSFnet-Relay.AC.UK>
To: w8sdz@WSMR-SIMTEL20.ARMY.MIL
Subject:   Patch for <MSDOS.DSKUTL>STUFIT.COM

Keith

STUFIT doesn't work for MSDOS 3.xx . Although it is dated 1988 in the archive
it was written in 1985. So one unknown is whether the author has himself
updated this.

For a while I thought that the author's description meant that files were
intended to go to the middle numbered sectors on the disk, since that is
what mostly happened. Then disassembly showed that the intention was to send
files to the highest numbered sectors still free.

So I wrote a patch, and my patched STUFIT now does what it was intended to
do. I attach the patch - it's hardly worth cutting off the code.

[ ... deleted ...]

Hylton

=========================================================================
STUFITUP.DBG - a patch for <msdos.dskutl>stufit.com

 Usage: DEBUG < STUFITUP.DBG
   (with STUFIT.COM in the default directory)

 Purpose: To patch STUFIT.COM for use with MSDOS 3.xx and 4.xx
 Status : Tested only with MSDOS 3.20, but follows the MSDOS intercept
          list for the use of INT21H/32H
 Comment: SIMTEL's <MSDOS.DSKUTL>STUFIT.ARC 6819/880811  originated in 1985,
          and assumes that MSDOS creates a file starting from the first
          available space: true for 2.xx, but not for 3.xx/4.xx.
          This is a simple patch to make that assumption temporarily true.
          It ensures that files go to their intended position in the
          highest numbered unused sectors.
 Comment: On an 8 MHz XT clone with a 20Mb ST-225, the STUFIT utility is
          slow but sure - about 12 sec a file for small files.
          There is clear scope for a re-write down to 7-8 sec a file
          for 3.xx and 4.xx.

Hylton Boothroyd, University of Warwick, 890710


and ...

from INTERRUP.LST v3-89

Here is the entry on INT 21h/32h as it currently exists on SIMTEL in the third
issue of the Interrupt list this year: INTER389

The information on
      DOS 3.x 1Eh
      DOS 4.0 1Fh
is new since INTER189. The rest looks unchanged.

Hylton

==========================================================================
INT 21 - DOS 2+ internal - GET DRIVE PARAMETER BLOCK
	AH = 32h
	DL = drive number
	    0 = default, 1 = A, etc.
Return: AL = 0FFh if invalid drive number, else
	DS:BX -> drive parameter block

Format of DOS Drive Parameter Block:
Offset	Size	Description
 00h	BYTE	drive number (0 = A, etc.)
 01h	BYTE	unit number within device driver
 02h	WORD	number of bytes per sector
 04h	BYTE	largest sector number in cluster (one less than sect/clust)
 05h	BYTE	log base two of the cluster size
 06h	WORD	number of reserved (boot) sectors
 08h	BYTE	number of copies of the FAT
 09h	WORD	number of root directory entries
 0Bh	WORD	first data sector on medium
 0Dh	WORD	largest possible cluster number (one more than # data clust)
---DOS 2.x---
 0Fh	BYTE	number of sectors in one FAT copy
 10h	WORD	first sector of root directory
 12h	DWORD	address of device driver for this drive
 16h	BYTE	media descriptor byte for medium
 17h	BYTE	FFh indicates block must be rebuilt
 18h	DWORD	address of next device block, offset = FFFFh indicates last
 1Ch	WORD	starting cluster of current directory (0 = root directory)
 1Eh 64 BYTEs	ASCIZ current directory path string
---DOS 3.x---
 0Fh	BYTE	number of sectors in one FAT copy
 10h	WORD	first sector of root directory
 12h	DWORD	address of device driver for this drive
 16h	BYTE	media descriptor byte for medium
 17h	BYTE	FFh = block must be rebuilt, 00h indicates block accessed
 18h	DWORD	address of next device block, offset = FFFFh indicates last
 1Ch	WORD	cluster at which to start search for free space when writing
 1Eh	WORD	number of free clusters on drive, FFFFh = unknown
---DOS 4.0---
 0Fh	WORD	number of sectors in one FAT copy
 11h	WORD	first sector of root directory
 13h	DWORD	address of device driver for this drive
 17h	BYTE	media descriptor byte for medium
 18h	BYTE	FFh = block must be rebuilt, 00h indicates block accessed
 19h	DWORD	address of next device block, offset = FFFFh indicates last
 1Dh	WORD	cluster at which to start search for free space when writing
 1Fh	WORD	number of free clusters on drive, FFFFh = unknown???

============================================================================
